package fr.inra.moulon.svgutils.svgspectrum;

/**
 *   SvgSpectrum free java library to draw annotated MS spectrum
 Copyright (C) 2009  Olivier Langella

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.


 * 
 * @author Olivier Langella <olivier.langella@moulon.inra.fr
 * 
 */

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import org.w3c.dom.*;

import fr.inra.moulon.svgutils.SvgGraphics;
import fr.inra.moulon.svgutils.SvgGroup;

public class SvgSpectrum implements fr.inra.moulon.svgutils.SvgCommon {

	private SvgGraphics svg_graphic;

	private SvgGroup spectrum_graph;

	// private Vector<Float> arr_mz;
	// private Vector<Float> arr_int;
	private float[] arr_mz;
	private float[] arr_int;
	private Vector<SvgIon> arr_ions = null;

	/**
	 * @brief associate peak index in the spectrum with SvgIon
	 * 
	 */
	private Hashtable<Integer, SvgIon> hash_assigned_peaks;

	// maximum intensity in this spectrum
	private float spectrumMaxIntensity;

	private float spectrum_width;
	private float spectrum_height;

	// min range to display spectrum
	private float spectrum_mz_min;
	// max range to display spectrum
	private float spectrum_mz_max;
	// max intensity inside the mz range
	private float spectrumRelativeMaxIntensity;

	private float transformYtranslate;

	private float transformYratio;

	private Float transformXtranslate;

	private float transformXratio;

	private float fitPrecision;
	private float delta = 0;

	/** threshold ratio : above this ratio, no need to print ion name */
	private float printThreshold;

	private boolean alreadyRendered;

	private SvgPeptideSequence sequence;
	private String title;

	float marginLeft;
	float marginRight;
	float marginBottom;
	float marginTop;

	/**
	 * @brief main class to produce spectrum SVG dom document
	 * 
	 * @param width
	 *            in pixels
	 * @param height
	 *            in pixels
	 * @throws Exception
	 */

	public SvgSpectrum(float width, float height) throws Exception {

		String visibilityHidden = "visibility:hidden;opacity:0;fill-opacity:0;stroke-opacity:0;";
		this.arr_mz = new float[0];
		this.arr_int = new float[0];

		alreadyRendered = false;
		this.fitPrecision = (float) 0.5;
		this.sequence = null;
		this.title = null;
		this.printThreshold = 0;
		this.spectrumMaxIntensity = 0;
		this.spectrumRelativeMaxIntensity = 0;

		hash_assigned_peaks = new Hashtable<Integer, SvgIon>();
		// super(width, height);
		svg_graphic = new SvgGraphics(width, height);
		svg_graphic.setCssClass("svg", "");
		svg_graphic
				.setCssClass(
						"*",
						"fill:none;fill-rule:evenodd;stroke:#000000;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1");
		// font-size:20px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:start;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Arial;-inkscape-font-specification:Arial
		/*
		 * svg_graphic .setCssClass( "text",
		 * "font-size:10px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:middle;fill:#000000;fill-opacity:1;stroke:none;stroke-width:0px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Arial;-inkscape-font-specification:Arial"
		 * );
		 */
		svg_graphic
				.setCssClass(
						"text",
						"font-size:10px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:middle;fill:#000000;stroke:none;stroke-width:0px;stroke-linecap:butt;stroke-linejoin:miter;font-family:Arial;-inkscape-font-specification:Arial");
		// tspan
		// {font-size:10px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;fill:#000000;fill-opacity:1;stroke:none;stroke-width:0px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Arial;-inkscape-font-specification:Arial}
		svg_graphic
				.setCssClass(
						"tspan",
						"font-size:10px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;fill:#000000;fill-opacity:1;stroke:none;stroke-width:0px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Arial;-inkscape-font-specification:Arial");
		svg_graphic.group("svgspectrumtitle", "");
		spectrum_graph = svg_graphic.group("spectrum", "");
		svg_graphic.goToGroup("spectrum");
		svg_graphic.group("peaks", "peak");
		svg_graphic.setCssClass(".peak *",
				"stroke:black;fill:black;fill-opacity:1;");

		SvgIonSpecies svgIon = SvgIonSpecies.getA1Instance();

		svg_graphic.group("a1", "");
		svg_graphic.setCssClass("#a1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#a1 text", visibilityHidden);
		svg_graphic.group("c1", "");
		svg_graphic.group("c2", "");
		svg_graphic.group("z1", "");
		svg_graphic.group("z2", "");
		svg_graphic.group("bO2", "");
		svgIon = SvgIonSpecies.getBO2Instance();
		svg_graphic.setCssClass("#bO2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#bO2 text", visibilityHidden);
		svgIon = SvgIonSpecies.getBO1Instance();
		svg_graphic.group("bO1", "");
		svg_graphic.setCssClass("#bO1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#bO1 text", visibilityHidden);
		svgIon = SvgIonSpecies.getBstar2Instance();
		svg_graphic.group("bstar2", "");
		svg_graphic.setCssClass("#bstar2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#bstar2 text", visibilityHidden);
		svgIon = SvgIonSpecies.getBstar1Instance();
		svg_graphic.group("bstar1", "");
		svg_graphic.setCssClass("#bstar1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#bstar1 text", visibilityHidden);
		svgIon = SvgIonSpecies.getYO2Instance();
		svg_graphic.group("yO2", "");
		svg_graphic.setCssClass("#yO2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#yO2 text", visibilityHidden);
		svgIon = SvgIonSpecies.getYO1Instance();
		svg_graphic.group("yO1", "");
		svg_graphic.setCssClass("#yO1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#yO1 text", visibilityHidden);
		svgIon = SvgIonSpecies.getYstar1Instance();
		svg_graphic.group("ystar1", "");
		svg_graphic.setCssClass("#ystar1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#ystar1 text", visibilityHidden);
		svgIon = SvgIonSpecies.getYstar2Instance();
		svg_graphic.group("ystar2", "");
		svg_graphic.setCssClass("#ystar2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass("#ystar2 text", visibilityHidden);
		svgIon = SvgIonSpecies.getB2Instance();
		svg_graphic.group("b2", "");
		svg_graphic.setCssClass("#b2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svgIon = SvgIonSpecies.getY2Instance();
		svg_graphic.group("y2", "");
		svg_graphic.setCssClass("#y2 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svgIon = SvgIonSpecies.getB1Instance();
		svg_graphic.group("b1", "b1");
		svg_graphic.setCssClass(".b1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass(".b1",
				"stroke-width:0px;stroke:" + svgIon.getCssColor() + ";fill:"
						+ svgIon.getCssColor() + ";fill-opacity:1;");
		// visibility: hidden
		svgIon = SvgIonSpecies.getY1Instance();
		svg_graphic.group("y1", "y1");
		svg_graphic.setCssClass(".y1 *", "stroke:" + svgIon.getCssColor()
				+ ";fill:" + svgIon.getCssColor() + ";fill-opacity:1;");
		svg_graphic.setCssClass(".y1",
				"stroke-width:0px;stroke:" + svgIon.getCssColor() + ";fill:"
						+ svgIon.getCssColor() + ";fill-opacity:1;");

	}

	public float getDelta() {
		return delta;
	}

	public void setDelta(float delta) {
		this.delta = delta;
	}

	/**
	 * \brief sets the ratio beyond which the ion label is not printed
	 * 
	 * @param printThresholdRatio
	 *            (value between 0 and 1)
	 * @throws Exception
	 */
	public void setPrintThreshold(float newPrintThreshold) throws Exception {
		if ((newPrintThreshold < 0) || ((newPrintThreshold > 1))) {
			throw new Exception("unable to set printThreshold of "
					+ newPrintThreshold
					+ ": it must be a value between 0 and 1");
		}
		this.printThreshold = newPrintThreshold;
	}

	public void setIonTable(Vector<SvgIon> arr_ions) {
		this.arr_ions = arr_ions;
	}

	public HashSet<SvgIon> getIonTable() {
		HashSet<SvgIon> arrReturn = new HashSet<SvgIon>();
		for (SvgIon ion : this.arr_ions) {
			arrReturn.add(ion);
		}
		return arrReturn;
	}

	private void assignPeaks() {
		for (SvgIon ion : this.arr_ions) {
			this.findBestPeakForIon(ion);
		}
		Set<Entry<Integer, SvgIon>> lesEntrees = this.hash_assigned_peaks
				.entrySet();
		Iterator<Entry<Integer, SvgIon>> it = lesEntrees.iterator();
		while (it.hasNext()) {

			Entry<Integer, SvgIon> e = (Map.Entry<Integer, SvgIon>) it.next();
			if (this.sequence != null) {
				this.sequence.addAssignedIon(e.getValue());
			}
		}

	}

	/**
	 * @brief return the hash table that associates peak indexes to svgions
	 * 
	 * @return Hashtable<Integer, SvgIon>
	 */
	Hashtable<Integer, SvgIon> getAssignedPeaks() {
		return this.hash_assigned_peaks;
	}

	private void findBestPeakForIon(SvgIon ion) {
		float mzmin = (float) (ion.getMz() - this.fitPrecision + delta);
		float mzmax = (float) (ion.getMz() + this.fitPrecision + delta);

		// float mindeviation = 99999;
		// float currentdeviation;
		float bestpeak_intensity = 0;
		Integer bestpeak = null;

		int i = 0;

		for (float curMz : this.arr_mz) {
			if (mzmin <= curMz) {
				if (curMz <= mzmax) {

					// currentdeviation = (this.arr_mz.get(i) - ion.getMz());
					// currentdeviation *= currentdeviation;
					// if (currentdeviation < mindeviation) {
					if (this.arr_int[i] > bestpeak_intensity) {
						bestpeak = i;
						bestpeak_intensity = this.arr_int[i];
						// mindeviation = currentdeviation;
					}
				}
			}
			i++;
		}

		if (bestpeak != null) {
			if (this.hash_assigned_peaks.containsKey(bestpeak)) {
				if (ion.getSvgIonSpecies().priority_over(
						this.hash_assigned_peaks.get(bestpeak)
								.getSvgIonSpecies())) {
					this.hash_assigned_peaks.get(bestpeak)
							.setAssignedPeakIntensityRatio(null);
					this.hash_assigned_peaks.put(bestpeak, ion);
					ion.setAssignedPeakIntensityRatio(this.arr_int[bestpeak]
							/ this.spectrumMaxIntensity);
				}
				return;
			}
			this.hash_assigned_peaks.put(bestpeak, ion);
			ion.setAssignedPeakIntensityRatio(this.arr_int[bestpeak]
					/ this.spectrumMaxIntensity);
		}
	}

	private void setAbsoluteSpectrumDescriptiveVars() {

		// **********************************
		// set margins
		this.updateMargin();

		spectrum_mz_min = this.arr_mz[0];
		spectrum_mz_max = spectrum_mz_min;

		if (this.arr_mz.length > 0) {
			spectrum_mz_max = this.arr_mz[this.arr_mz.length - 1];
		}

		this.autoScaleIntensity();
		setXratio();

		this.spectrumMaxIntensity = this.spectrumRelativeMaxIntensity;

		// spectrum_max_intensity = this
	}

	/**
	 * @brief set mz and intesity for this spectrum
	 * 
	 * @param arr_d_mz
	 * @param arr_d_int
	 */
	public void setMzIntensity(ArrayList<Double> varr_mz,
			ArrayList<Double> varr_int) {
		if (varr_mz == null) {
			return;
		}
		this.arr_mz = new float[varr_mz.size()];
		this.arr_int = new float[varr_int.size()];
		int i = 0;
		for (Double mz : varr_mz) {
			this.arr_mz[i] = mz.floatValue();
			this.arr_int[i] = varr_int.get(i).floatValue();
			// System.out.println(this.arr_int[i]);
			i++;
		}
		setAbsoluteSpectrumDescriptiveVars();
	}

	public void setMzIntensity(Vector<Float> varr_mz, Vector<Float> varr_int) {
		if (varr_mz == null) {
			return;
		}
		this.arr_mz = new float[varr_mz.size()];
		this.arr_int = new float[varr_int.size()];
		int i = 0;
		for (Float mz : varr_mz) {
			this.arr_mz[i] = mz;
			this.arr_int[i] = varr_int.get(i);
			// System.out.println(this.arr_int[i]);
			i++;
		}
		setAbsoluteSpectrumDescriptiveVars();
	}

	public void setMzIntensity(double[] in_arr_mz, double[] in_arr_int) {

		if (arr_mz == null) {
			return;
		}
		this.arr_mz = new float[in_arr_mz.length];
		this.arr_int = new float[in_arr_int.length];
		int i = 0;
		for (double mz : in_arr_mz) {
			this.arr_mz[i] = (float) mz;
			this.arr_int[i] = (float) in_arr_int[i];
			// System.out.println(this.arr_int[i]);
			i++;
		}
		setAbsoluteSpectrumDescriptiveVars();
	}

	public void setMzIntensity(float[] arr_mz2, float[] arr_int2) {
		this.arr_mz = arr_mz2;
		this.arr_int = arr_int2;
		setAbsoluteSpectrumDescriptiveVars();

	}

	private void drawPeaks() throws Exception {

		if (alreadyRendered) {
			return;
		}
		alreadyRendered = true;

		if (this.arr_ions != null) {
			this.assignPeaks();
		}

		// **********************************
		// set margins
		this.updateMargin();

		// **********************************
		// draw peaks

		svg_graphic.goToGroup("peaks");
		int i = 0;
		for (float curMz : this.arr_mz) {
			if ((curMz >= this.spectrum_mz_min)
					&& (curMz <= this.spectrum_mz_max)) {

				Float intensity = this.arr_int[i];
				if (this.hash_assigned_peaks.containsKey(i)) {

					svg_graphic.goToGroup(this.hash_assigned_peaks.get(i)
							.getSvgIonSpecies().getCssClassname());

					// Element ionlabel =
					if ((intensity / this.spectrumRelativeMaxIntensity) > printThreshold) {
						this.svg_graphic.text(this.mzToX(curMz),
								this.intToY(this.arr_int[i]) - 5,
								this.hash_assigned_peaks.get(i).getName(),
								this.hash_assigned_peaks.get(i)
										.getSvgIonSpecies().getCssClassname());
					}

					// ionlabel.setAttribute("transform",
					// "rotate(-45 "+ionlabel.getAttribute("x")+" "+ionlabel.getAttribute("y")+")");

				} else {
					svg_graphic.goToGroup("peaks");
				}
				this.svg_graphic.line(this.mzToX(curMz), this.intToY(0),
						this.mzToX(curMz), this.intToY(intensity), "");
			}

			i++;
		}

		// **********************************
		// write axis

		svg_graphic.goToGroup("spectrum");
		// Y axis :
		svg_graphic.line(-1, 0, -1, spectrum_height, "");
		// X axis :
		svg_graphic.line(-1, spectrum_height, spectrum_width + 1,
				spectrum_height, "");

		// **********************************
		// write title

		if (this.title != null) {
			svg_graphic.goToGroup("svgspectrumtitle");
			svg_graphic.text((spectrum_width / 2) + this.marginLeft, 30, title,
					"");
			svg_graphic.setCssClass("#svgspectrumtitle tspan",
					"font-size:22px;stroke:#000000;fill:#000000;");
		}

		// **********************************
		// write sequence

		if (this.sequence != null) {
			svg_graphic.goToGroup("spectrum");
			this.sequence.drawSvgPeptideSequence(this.svg_graphic,
					((spectrum_width / 2) - (sequence.svgWidth()) / 2), -30);
			// svg_graphic.text((spectrum_width / 2) - (((sequence.size()-1) *
			// 20)/2), 0, sequence.toString(), "",20);
		}

		// **********************************
		// write interval count on X axis
		svg_graphic.goToGroup("spectrum");
		svg_graphic.text(this.spectrum_width / 2, spectrum_height + 25, "m/z",
				"");

		int big_tic_int = 10000;

		float delta = this.spectrum_mz_max - this.spectrum_mz_min;
		while (big_tic_int > 10) {
			if ((delta / big_tic_int) > 2) {
				break;
			}
			big_tic_int = big_tic_int / 10;
		}
		if (big_tic_int >= 10) {
			int small_tic_int = (int) Math.floor(big_tic_int / 10);

			int begin_tic = (int) ((Math.floor(this.spectrum_mz_min
					/ big_tic_int) + 1) * big_tic_int);

			for (i = begin_tic; i < this.spectrum_mz_max; i += big_tic_int) {
				// draw
				String label = "" + i;
				float pt_x = this.mzToX(i);
				// draw a line
				svg_graphic.line(pt_x, spectrum_height, pt_x,
						spectrum_height + 5, "");
				// imageline($image, $pt_x, $y_end, $pt_x, $y_end +5, $grey);

				svg_graphic.text(pt_x, spectrum_height + 15, label, "");
				// imagestring($image, 4, $pt_x - (strlen($string) *
				// $this->_char_width) / 2, $y_end + ($this->_char_height *
				// 0.5), $string, $black);

			}

			begin_tic = (int) ((Math
					.floor(this.spectrum_mz_min / small_tic_int) + 1) * small_tic_int);

			for (i = begin_tic; i < this.spectrum_mz_max; i += small_tic_int) {
				// draw
				float pt_x = this.mzToX(i);
				// $pt_x = $x_start + ($x_end - $x_start) * ($i - $min_x) /
				// ($max_x - $min_x);
				svg_graphic.line(pt_x, spectrum_height, pt_x,
						spectrum_height + 2, "");
				// imageline($image, $pt_x, $y_end, $pt_x, $y_end +2, $grey);
			}
		}
		// end write interval count on Y axis
		// *************************

		// **********************************
		// write interval count on Y axis
		svg_graphic.goToGroup("spectrum");
		// exponential
		// svg_graphic.setCssClass("tspan.exponential","baseline-shift:super;");
		svg_graphic.setCssClass("text.yaxislabel", "text-anchor:end;");

		Element ylabel = svg_graphic.text(-30, spectrum_height / 2,
				"intensity", "");
		ylabel.setAttribute("transform", "rotate(270 -30," + spectrum_height
				/ 2 + ")");

		big_tic_int = 1000000;

		delta = this.spectrumRelativeMaxIntensity - 0;
		while (big_tic_int > 10) {
			if ((delta / big_tic_int) > 2) {
				break;
			}
			big_tic_int = big_tic_int / 10;
		}
		DecimalFormat formScientific = (DecimalFormat) NumberFormat
				.getIntegerInstance();
		formScientific.applyPattern("##0E0");
		DecimalFormat formClassic = (DecimalFormat) NumberFormat
				.getIntegerInstance();
		// formClassic.applyPattern("##0E0");
		if (big_tic_int >= 10) {
			int small_tic_int = (int) Math.floor(big_tic_int / 10);

			float begin_tic = (int) ((Math.floor(0 / big_tic_int) + 1) * big_tic_int);

			for (Float ipos = begin_tic; ipos < this.spectrumRelativeMaxIntensity; ipos += big_tic_int) {
				// draw
				String label = "";
				if (ipos < 10001) {
					label = formClassic.format(ipos);
				} else {
					label = formScientific.format(ipos);
				}
				float pt_y = this.intToY(ipos);
				// draw a line
				svg_graphic.line(-1, pt_y, -6, pt_y, "");

				svg_graphic.scientificNumber(-7, pt_y + 3, label, "yaxislabel");

			}

			begin_tic = (int) ((Math.floor(0 / small_tic_int) + 1) * small_tic_int);

			for (Float ipos = begin_tic; ipos < this.spectrumRelativeMaxIntensity; ipos += small_tic_int) {
				// draw
				float pt_y = this.intToY(ipos);
				svg_graphic.line(-1, pt_y, -3, pt_y, "");
			}
		}
		// end write interval count on Y axis
		// *************************

	}

	private void updateMargin() {
		marginLeft = (float) (60);
		marginRight = (float) (30);
		// float hmargin = (float) (height * 0.05);
		marginBottom = (float) (30);
		marginTop = (float) (30);
		if (this.sequence != null) {
			marginTop += 40;
		}
		if (this.title != null) {
			marginTop += 30;
		}

		spectrum_height = this.svg_graphic.getHeight()
				- (marginBottom + marginTop);
		spectrum_width = this.svg_graphic.getWidth()
				- (marginLeft + marginRight);

		float ratiox = 1;
		float ratioy = 1;
		// 1.0,0.0,0.001,1.0,60.0,70.0
		/*
		 * spectrum_graph.transformMatrix(ratiox, 0, 0, ratioy, marginLeft,
		 * marginTop);
		 */
		spectrum_graph.transformMatrix(ratiox, 0, (float) 0.001, ratioy,
				marginLeft, marginTop);

		autoScaleIntensity();

	}

	private float intToY(float intensity) {
		intensity = intensity + this.transformYtranslate;
		intensity = intensity * this.transformYratio;
		return (spectrum_height - intensity);
	}

	private float mzToX(float mz) {
		mz = mz + this.transformXtranslate;
		mz = mz * this.transformXratio;
		return mz;
	}

	private float getRelativeMaxIntensityValue() {
		float maxIntensity = 0;
		if (this.arr_int.length == 0) {
			maxIntensity = 0;
		} else {
			for (int i = 0; i < arr_int.length; i++) {
				if (this.arr_mz[i] >= this.spectrum_mz_min) {
					if (this.arr_mz[i] <= this.spectrum_mz_max) {
						if (arr_int[i] > maxIntensity) {
							maxIntensity = arr_int[i];
						}
					}
				}
			}
		}
		return maxIntensity;
	}

	private void autoScaleIntensity() {

		this.spectrumRelativeMaxIntensity = getRelativeMaxIntensityValue();

		this.transformYtranslate = 0;
		this.transformYratio = this.spectrum_height
				/ spectrumRelativeMaxIntensity;

	}

	public void setSpectrumRangeMz(float mzMin, float mzMax) {
		spectrum_mz_min = mzMin;
		spectrum_mz_max = mzMax;

		// this.updateMargin();
		this.autoScaleIntensity();
		this.setXratio();
	}

	private void setXratio() {
		this.transformXtranslate = -spectrum_mz_min;
		this.transformXratio = this.spectrum_width
				/ (spectrum_mz_max - spectrum_mz_min);
	}

	/**
	 * @brief write SVG document to file
	 * 
	 */

	public void toFile(File filename) throws Exception {

		try {
			// Open an output stream
			FileOutputStream fstream = new FileOutputStream(
					filename.getAbsolutePath());
			// BufferedWriter out = new BufferedWriter(fstream);
			// Print a line of text
			this.toOutputStream(fstream);

			// Close our output stream
			fstream.close();
		}
		// Catches any error conditions
		catch (IOException e) {
			System.err.println("Unable to write to file");
			System.exit(-1);
		}

	}

	/**
	 * @brief write SVG document to output stream
	 * 
	 */
	public void toOutputStream(OutputStream out) throws Exception {

		this.drawPeaks();
		svg_graphic.toOutputStream(out);
	}

	public Document getDocument() throws Exception {
		this.drawPeaks();
		return this.svg_graphic.getDocument();
	}

	public void setSequence(SvgPeptideSequence monPeptide) {
		this.sequence = monPeptide;
		this.setIonTable(monPeptide.getIonTable());
	}

	public void setFitPrecision(float fitPrecision) {
		this.fitPrecision = fitPrecision;
	}

	public float getFitPrecision() {
		return fitPrecision;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getTitle() {
		return title;
	}

	/**
	 * @return the hash_assigned_peaks
	 */
	public Hashtable<Integer, SvgIon> getHash_assigned_peaks() {
		return hash_assigned_peaks;
	}

	public Element getSvgRoot() {
		return this.svg_graphic.getSvgRoot();
	}

	public SvgPeptideSequence getSvgPeptideSequence() {
		return this.sequence;
	}

	public Integer getAssignedPeak(SvgIon ion) {

		for (Integer peakNum : hash_assigned_peaks.keySet()) {
			if (hash_assigned_peaks.get(peakNum).equals(ion)) {
				return peakNum;
			}
		}
		return null;
	}

	public HashSet<SvgAaMod> getSvgAaModSet() {
		return this.sequence.getSvgAaModSet();
	}

}
